@@ -1,19 +1,25 @@
 import logging
-from typing import Dict, Optional
+from typing import Dict, Optional, List
+
+from requests import Response
 
 from dynatrace.activegate import ActiveGate
-from dynatrace.http_client import HttpClient
-from dynatrace.entity import Entity
+from dynatrace.dashboard import DashboardStub, Dashboard
+from dynatrace.endpoint import EndpointShortRepresentation
+from dynatrace.entity import Entity, EntityShortRepresentation
 from dynatrace.entity_type import EntityType
+from dynatrace.extension import ExtensionDto, Extension, ExtensionShortRepresentation, ExtensionConfigurationDto
+from dynatrace.http_client import HttpClient
 from dynatrace.metric import MetricSeriesCollection, MetricDescriptor
 from dynatrace.pagination import PaginatedList
-
-default_log = logging.getLogger("dynatrace")
+from dynatrace.plugins import PluginShortRepresentation, PluginState
 
 
 class Dynatrace:
-    def __init__(self, base_url: str, token: str, log: logging.Logger = default_log, proxies: Dict = None):
-        self.__http_client = HttpClient(base_url, token, log, proxies)
+    def __init__(
+        self, base_url: str, token: str, log: logging.Logger = None, proxies: Dict = None, too_many_requests_strategy=None
+    ):
+        self.__http_client = HttpClient(base_url, token, log, proxies, too_many_requests_strategy)
 
     def get_entities(
         self, entity_selector: str, time_from: str = "now-2h", time_to: str = "now", fields: Optional[str] = None, page_size=50
@@ -287,7 +293,7 @@ class Dynatrace:
             The length of the string is limited to 1,000 characters.
         :return: The metric descriptor
         """
-        response = self.__http_client.make_request(f"/api/v2/metrics/{metric_id}", None, None).json()
+        response = self.__http_client.make_request(f"/api/v2/metrics/{metric_id}").json()
         return MetricDescriptor(self.__http_client, None, response)
 
     def get_activegates(
@@ -343,3 +349,102 @@ class Dynatrace:
             "version": version,
         }
         return PaginatedList(ActiveGate, self.__http_client, "/api/v2/activeGates", params, list_item="activeGates")
+
+    def get_plugins(self) -> PaginatedList[PluginShortRepresentation]:
+        """
+        List all uploaded plugins
+        """
+        return PaginatedList(PluginShortRepresentation, self.__http_client, "/api/config/v1/plugins", list_item="values")
+
+    def get_plugin_states(self, plugin_id) -> PaginatedList[PluginState]:
+        """
+        List the states of the specified plugin
+        """
+        return PaginatedList(PluginState, self.__http_client, f"/api/config/v1/plugins/{plugin_id}/states", list_item="states")
+
+    def delete_plugin(self, plugin_id) -> Response:
+        """
+        Deletes the ZIP file of the specified plugin
+        :param plugin_id: The ID of the plugin to be deleted
+        """
+        return self.__http_client.make_request(f"/api/config/v1/plugins/{plugin_id}/binary", method="DELETE")
+
+    def get_endpoints(self, plugin_id: str) -> PaginatedList[EndpointShortRepresentation]:
+        """
+        Lists endpoints of the specified ActiveGate plugin
+        """
+        return PaginatedList(
+            EndpointShortRepresentation, self.__http_client, f"/api/config/v1/plugins/{plugin_id}/endpoints", list_item="values"
+        )
+
+    def get_dashboards(self, owner: str = None, tags: List[str] = None) -> PaginatedList[DashboardStub]:
+        """
+        Lists all dashboards of the environment
+        :param owner: The owner of the dashboard.
+        :param tags: A list of tags applied to the dashboard.
+            The dashboard must match all the specified tags.
+        """
+        params = {"owner": owner, "tags": tags}
+        return PaginatedList(DashboardStub, self.__http_client, f"/api/config/v1/dashboards", params, list_item="dashboards")
+
+    def delete_dashboard(self, dashboard_id: str) -> Response:
+        """
+        Deletes the specified dashboard
+        """
+        return self.__http_client.make_request(f"/api/config/v1/dashboards/{dashboard_id}", method="DELETE")
+
+    def get_dashboard(self, dashboard_id: str) -> Dashboard:
+        """
+        Gets the properties of the specified dashboard
+        """
+        response = self.__http_client.make_request(f"/api/config/v1/dashboards/{dashboard_id}").json()
+        return Dashboard(self.__http_client, None, response)
+
+    def get_extensions(self, page_size: int = 200) -> PaginatedList[ExtensionDto]:
+        """
+        List all uploaded extensions
+
+        :param page_size: The number of results per result page. Must be between 1 and 500
+            Default value : 200
+        """
+        params = {"pageSize": page_size}
+        return PaginatedList(ExtensionDto, self.__http_client, f"/api/config/v1/extensions", params, list_item="extensions")
+
+    def get_extension(self, extension_id: str):
+        response = self.__http_client.make_request(f"/api/config/v1/extensions/{extension_id}").json()
+        return Extension(self.__http_client, None, response)
+
+    def get_extension_instances(self, extension_id: str, page_size: int = 200) -> PaginatedList[ExtensionShortRepresentation]:
+        params = {"pageSize": page_size}
+        return PaginatedList(
+            ExtensionShortRepresentation,
+            self.__http_client,
+            f"/api/config/v1/extensions/{extension_id}/instances",
+            list_item="configurationsList",
+            target_params=params,
+        )
+
+    def get_extension_instance(self, extension_id: str, configuration_id: str):
+        response = self.__http_client.make_request(
+            f"/api/config/v1/extensions/{extension_id}/instances/{configuration_id}"
+        ).json()
+        return ExtensionConfigurationDto(self.__http_client, None, response)
+
+    def post_extension_instance(self, extension_configuration: ExtensionConfigurationDto):
+        return extension_configuration.post()
+
+    def create_extension_instance(
+        self,
+        extension_id: str,
+        enabled: bool = True,
+        use_global: bool = True,
+        properties: dict = None,
+        host_id: str = None,
+        active_gate: EntityShortRepresentation = None,
+        endpoint_id: str = None,
+        endpoint_name: str = None,
+    ) -> ExtensionConfigurationDto:
+
+        return ExtensionConfigurationDto(
+            self.__http_client, extension_id, enabled, use_global, properties, host_id, active_gate, endpoint_id, endpoint_name
+        )
